**********************************************************
*                                                        *
*                    Change History                      *
*                                                        *
*   05/03/10:                                            *
*                                                        *
*   Fixed &TIMEOUT to return to AMPERSAND to set error   *
*   flags (0) and (1) properly.                          *
*                                                        *
*   04/28/10:                                            *
*                                                        *
*   Removed version 2.x code from BOOTREQ.               *
*                                                        *
*   Added PEEKPOKE request/server for atomic semaphores. *
*                                                        *
*   Changed JSR to service routine in AMPERSAND to go    *
*   through an indirect JMP to avoid code modification.  *
*                                                        *
*   Factored out error retry code for simple requests    *
*   (SIMPLREQ) to save space in CALLREQ.                 *
*                                                        *
*   02/10/09 === Released NadaNet 3.0                    *
*                                                        *
*   01/24/09:                                            *
*                                                        *
*   Added ONERR ($D8) definition and code to clear the   *
*   flag at boot time and at &RUN time.                  *
*                                                        *
*   01/06/09:                                            *
*                                                        *
*   Modified NADA.CRATE's NADABOOT2 to coldstart BASIC   *
*   and set HIMEM to base of NadaNet.                    *
*                                                        *
*   Changed 'version' to a 2-digit BCD value that is     *
*   used in messages and the version byte.               *
*                                                        *
*   11/11/08:                                            *
*                                                        *
*   Modified RUNSRV to save and restore CSW/KSW hooks    *
*   so that &RUN works properly with or without an OS.   *
*                                                        *
*   11/03/08:                                            *
*                                                        *
*   Added call to FIXLINKS into RUNSRV code to allow     *
*   BASIC programs to be &RUN at any address > $800.     *
*                                                        *
*   10/06/08:                                            *
*                                                        *
*   Added simple BCAST action to SERVER that just sets   *
*   'address' and 'length' to request values and then    *
*   returns to the calling code to deal with the data.   *
*                                                        *
*   Added table of BCAST tags to NADADEFS to serve as a  *
*   central directory of BCAST tag values.               *
*                                                        *
*   09/25/08:                                            *
*                                                        *
*   Added &RUN and &BRUN, as derivatives of &POKE, to    *
*   run Applesoft programs and M/L programs.             *
*                                                        *
*   Added RCVCTL, RCVPTR, rarl=>al, and RCVLONG to the   *
*   entry point vector for use by BCAST server code.     *
*                                                        *
*   09/04/08:                                            *
*                                                        *
*   Restructured SERVER to correct failure to re-sync if *
*   'reqctr' was satisfied, and to minimze "deaf" time   *
*   when iterating in SERVER.                            *
*                                                        *
*   Added 'reqpidle' (requests per idletime) definition, *
*   which is closer to typical.                          *
*                                                        *
*   08/20/08:                                            *
*                                                        *
*   Added BCAST request as a general mechanism for       *
*   broadcasting data.                                   *
*                                                        *
*   Added BCASTARB to arbitrate and lock net, then delay *
*   for 20ms. to allow all collisions to resolve and any *
*   "slow" pollers to get into their RCVCTL holds.  This *
*   arbitration will precede all broadcast requests, like*
*   BOOT, BCAST, and BPOKE.                              *
*                                                        *
*   08/16/08:                                            *
*                                                        *
*   Changed control packet format:                       *
*   - Combined 'req' and 'mod' bytes into 'rqmd' byte.   *
*   - Added complement of 'frm', called 'frmc', as a way *
*     to detect collisions of synchronized packets.      *
*   - Removed "delayed BOOTREQ after GETID" boot protocol*
*   - Modified BOOTREQ to send old format packet for     *
*     compatibility with v2.1 PassiveBoot ROM.           *
*   - Changed sign-on version to v3.0.                   *
*                                                        *
*   Moved error counters to just after IDTBL, and        *
*   prefixed them with NadaNet version in hex.           *
*                                                        *
*   07/25/08 === Released NadaNet 2.1                    *
*                                                        *
*   05/21/08:                                            *
*                                                        *
*   Made numerous significant space optimizations:       *
*   - Added subroutines to set 'address' & 'length'      *
*     from most common variables.                        *
*   - Added PROTERR subroutine to increment count.       *
*   - Put checksum counting inside RCVPKT.               *
*   - Added variable delay preceding SENDLONG.           *
*   - Placed all of the above in "slack" space following *
*     page-aligned SENDPKT and RCVPKT.                   *
*   - Deleted MONITOR (can always load when needed).     *
*                                                        *
*   Fixed potential bug if INSTALL called >255 times.    *
*                                                        *
*   Added ID error checking to 'setid' and INIT.         *
*                                                        *
*   Changed broadcast BOOTREQ to not be protocol error.  *
*                                                        *
*   Added 'xmain', 'xsend', and 'xreceive' symbols to    *
*   make slack space easy to read in symbol table.       *
*                                                        *
*   04/21/08:                                            *
*                                                        *
*   Added 'svrxkbd' entry to SERVER, used by 'servelp'   *
*   to ignore keyboard input.                            *
*                                                        *
*   04/15/08:                                            *
*                                                        *
*   Split NADADEFS include file into three parts so that *
*   the control packet definitions could be used without *
*   generating code for the vectors and variables.       *
*                                                        *
*   Added new "Enhanced boot" protocol for AppleCrate    *
*   machines using Enhanced //e's.  The new protocol     *
*   blindly broadcasts the boot code whenever &BOOTCODE  *
*   is invoked.                                          *
*                                                        *
*   Since only RCVPKT and RCVLONG, plus RESET and the    *
*   actual boot logic need reside in ROM, it fits easily *
*   into the 2 pages of code used by the Enhanced //e's  *
*   self-test code.                                      *
*                                                        *
*   To support this, a new conditional assembly flag,    *
*   'enhboot' has been added and used to select the      *
*   code in SENDRCV and NADADEFS for the new boot ROM.   *
*                                                        *
*   The new AppleCrate boot image will be prefixed with  *
*   a second-stage boot that will use GETID to allocate  *
*   unique machine IDs.  The DACK length hi-byte sent by *
*   Enhanced machines contains a "magic number" ($A5) to *
*   signal that GETID need not schedule a boot code send.*
*                                                        *
*   The second stage boot will set up the page 3 RESET   *
*   vector to go directly to NadaNet INIT, so it does    *
*   not take up space in the running machine.            *
*                                                        *
*   The second-stage boot code uses a non-zero 'bootself'*
*   value to indicate that a GETID has already been done *
*   and the second-stage can be skipped for unenhanced   *
*   AppleCrate machines.                                 *
*                                                        *
*   The maximum number of machines has been increased to *
*   31, and the zeroth entry in the IDTABLE is 31.       *
*                                                        *
*   07/21/05 === Released NadaNet 2.0                    *
*                                                        *
*   06/29/05:                                            *
*                                                        *
*   Added NadaNet version 2.0 sign-on message to INIT.   *
*                                                        *
*   Replaced tree-like data send routing in SENDPKT with *
*   new, shorter, lattice-like routine created by Stephen*
*   Thomas.  The new routine sends all bits in uniform   *
*   cells of 8 cycles, lowering the cycles/byte to 95    *
*   from 106, for a speed increase of more than 11%.     *
*                                                        *
*   The new data transfer rate is over 10 KB/second.     *
*                                                        *
*   The packet start synchronization now allows RCVPKT   *
*   to establish fine sync for the first byte.           *
*                                                        *
*   The new RCVPKT samples data bitcells only during the *
*   5th, 6th, and 7th of 8 cycles, making NadaNet much   *
*   more tolerant of long network time constants caused  *
*   by too much cable or too high a pulldown resistance. *
*                                                        *
*   The timing of the check byte is now identical to the *
*   timing of data bytes.                                *
*                                                        *
*   RCVPKT is also changed to reflect the new timings,   *
*   which required unrolling the receive code again.     *
*                                                        *
*   Increased receive-to-send turnaround delays in       *
*   PEEKSRV and GETMSRV to allow some margin for the     *
*   receiving machine to start polling.                  *
*                                                        *
*   06/08/05:                                            *
*                                                        *
*   Fixed bus fight in RCVPKT pointed out by an astute   *
*   reader of the code, Stephen Thomas, who also sent    *
*   replacement code which only reads the paddle input   *
*   and which cleverly combines data shifting with loop  *
*   control, permitting the receive code to be re-rolled *
*   to save space!                                       *
*                                                        *
*   12/05/04 === Released NadaNet 1.2                    *
*                                                        *
*   12/01/04:                                            *
*                                                        *
*   Fixed GETID bug that left 'sbuf+adr' unset if the ID *
*   received was not a temporary ID.  (For masters only) *
*                                                        *
*   Parameterized maximum number of machines (maxid) and *
*   changed GETID so that any ID > maxid is considered a *
*   temporary ID to be assigned a permanent ID.          *
*                                                        *
*   Changed handling of protocol errors in SERVER so     *
*   they are "timed" as if they were requests.           *
*                                                        *
*   11/17/04:                                            *
*                                                        *
*   Changed 'servegap' wait time to 3/4 of min arb time  *
*   to allow some margin for server routine processing   *
*   after network is released (which is subtracted from  *
*   "SERVER visible" inter-request gap).                 *
*                                                        *
*   11/13/04                                             *
*                                                        *
*   Changed AmperNada handler to leave return variable   *
*   unchanged when an error occurs.                      *
*                                                        *
*   11/12/04:                                            *
*                                                        *
*   Increased BPOKE locked wait time to 20ms. to allow   *
*   more "dead time" in an Applesoft &SERVE polling loop.*
*                                                        *
*   11/10/04:                                            *
*                                                        *
*   Changed SERVER gap wait to wait for an unchanging    *
*   net, not an idle net, so that a BPOKE is received    *
*   when preceded by a locked state.                     *
*                                                        *
*   Fixed bug in PKINCSRV that dropped carry.            *
*                                                        *
*   11/08/04:                                            *
*                                                        *
*   Changed AmperNada handler to throw an Applesoft      *
*   "DATA" (49) error by default when a command fails.   *
*   This error can be caught by an active ONERR, or it   *
*   can be suppressed by appending a "#" to the command. *
*   If the error is suppressed, it is the programmer's   *
*   responsibility to check status by PEEKing 1 and 0.   *
*                                                        *
*   11/06/04:                                            *
*                                                        *
*   Removed &ONERR(err?) because its residual effects--  *
*   storing status into variable memory--outlast any     *
*   running program unless explicitly cancelled.  Using  *
*   PEEK(1) is a safe and effective alternative solution.*
*                                                        *
*   11/05/04:                                            *
*                                                        *
*   Added BPOKE & PEEKINC requestors and servers.        *
*                                                        *
*   Added &IDTBL(val?) to retrieve address of 'idtable'  *
*   in 'master' version.                                 *
*                                                        *
*   Changed ARBTRATE to use a single loop, and SETID to  *
*   use ID for 'arbxv' when a temp ID (>$7F) is used.    *
*                                                        *
*   11/01/04:                                            *
*                                                        *
*   Integrated AmperNada ampersand interface for BASIC   *
*   into NadaNet.  Size limit is now $900.               *
*                                                        *
*   10/27/04:                                            *
*                                                        *
*   Fixed latent BOOT timing bug in server.              *
*                                                        *
*   Changed SERVER so that it returns after processing   *
*   any request, in addition to when a key is pressed.   *
*                                                        *
*   Changed SERVER and CALLSRV to do indirect jumps,     *
*   rather than pushing addresses on stack for rts.      *
*                                                        *
*   Changed REQUEST resend count so that the request     *
*   timeout set by 'reqtime' is accurate.                *
*                                                        *
*   Changed MONITOR to wait for a minimum period of      *
*   unchanging network state, rather than '0' state, so  *
*   a locked state between packets is detected as a gap. *
*                                                        *
*   10/20/04:                                            *
*                                                        *
*   Added changes so that NADABOOT could be built using  *
*   standard NADANET "put" files.                        *
*                                                        *
*   Split this change history into a separate file.      *
*                                                        *
*   10/18/04:                                            *
*                                                        *
*   Added code to INIT to set up $3CD with warm start    *
*   'JMP servelp', so $3CF is NADANET's load page.       *
*                                                        *
*   10/13/04:                                            *
*                                                        *
*   Made PUTMREQ and GETMREQ conditional upon 'master'   *
*   conditional compile flag.  PUTMSRV and GETMSRV are   *
*   conditional upon 'not master'.  This frees up space  *
*   for additional enhancements by splitting NadaNet into*
*   different functional subsets for different purposes. *
*                                                        *
*   Made RCVPKT timeout variable so it can be set to the *
*   minimum arbitration time while within a protocol,    *
*   to protect the protocol from "outside" interference, *
*   and set to 20ms. outside a protocol, when SERVE or   *
*   MONITOR is running, to reduce polling dead time.     *
*                                                        *
*   Moved packet-starting 'ONE' earlier in SENDPKT so    *
*   that ARBTRATE and RCVPKT do not need to double-poll  *
*   bus to detect start pulse.                           *
*                                                        *
*   Changed BOOTREQ to use boot code address, length,    *
*   and local address set up prior to SERVER call.       *
*                                                        *
*   Split entry point vector and variable definitions    *
*   out into NADADEFS "put" file for use in other progs. *
*                                                        *
*   10/05/04:                                            *
*                                                        *
*   Changed ARBTRATE to lock the bus after a successful  *
*   poll, so that the arbitration "increment" could be   *
*   reduced to 22 cycles from 66.                        *
*                                                        *
*   Changed SETID arbitration time calculation to match. *
*                                                        *
*   Changed SERVER so that received requests, whether    *
*   acted upon or not, are counted as 1/8 of a 20ms.     *
*   timeout interval.  This allows time-related events   *
*   to occur properly whether the net is busy or idle.   *
*                                                        *
*   Moved 'sbuf', 'rbuf', and counters so that they will *
*   move less in the future.                             *
*                                                        *
*   Made REQUEST retry limit an initialized variable so  *
*   that it can be lowered to speed up detection of a    *
*   possibly non-existent machine.                       *
*                                                        *
*   Moved the 'monch' table used by PUTMSRV and GETMSRV  *
*   from internal memory to the unused top of the page   *
*   map table, saving 48 bytes of program memory.        *
*                                                        *
*   07/26/04 === Released NadaNet 1.1                    *
*                                                        *
*   06/23/04:                                            *
*                                                        *
*   Added iteration counter to SERVER and dispensed with *
*   SERVE1.  Changed SERVE sync wait to min arbitration  *
*   time.                                                *
*                                                        *
*   Disabled interrupts during SENDPKT and RCVPKT.       *
*                                                        *
*   Made "packet"/"message" nomenclature consistent.     *
*                                                        *
*   06/04/04:                                            *
*                                                        *
*   Made INIT, SERVE, and BOOT functions and PEEK, POKE, *
*   and CALL functions separate include modules.         *
*                                                        *
*   Added text graphics for protocols.                   *
*                                                        *
*   05/26/04                                             *
*                                                        *
*   Added MONITOR function for snooping all packets on   *
*   the network and logging the first 8 bytes in memory. *
*                                                        *
*   05/15/04:                                            *
*                                                        *
*   Added "master" conditional assembly switch to        *
*   control newly added  boot functions, BOOTREQ and     *
*   GETIDSRV.                                            *
*                                                        *
*   05/06/04:                                            *
*                                                        *
*   Shortened arbitration time to 1 ms., since almost    *
*   all protocols have less than 1 ms. delay between     *
*   packets.  Exceptions will "lock" the net by pulling  *
*   it high until they can respond.  This is effectively *
*   an extended "start" pulse, and RCVPKT will wait      *
*   indefinitely for the transition to low.              *
*                                                        *
*   Currently, only PUTMSRV and GETMSRV can take longer  *
*   than 1 ms. to respond with ACK or NAK, so they must  *
*   lock the net until their response.                   *
*                                                        *
*   The delay from last arbitration poll until beginning *
*   of "start" pulse is 54 cycles, so to avoid collision *
*   the arbitration delay difference between machines    *
*   must exceed 54.  Since it must be a multiple of 11   *
*   cycles, the offset is machine ID * 66 cycles.        *
*                                                        *
*   Since 8-byte data packets are indistinguishable      *
*   from control packets, and since data packets are     *
*   never delayed from a preceding control packet by     *
*   more than 0.5 ms., SERVER must wait for a net "idle" *
*   (low) state for 0.5 ms. to ensure that the next pkt  *
*   it receives is not data.  This delay is only needed  *
*   if the net has not been polled for more than the     *
*   arbitration delay (~1 ms.).                          *
*                                                        *
*   04/19/04:                                            *
*                                                        *
*   Changed RCVPKT timeout to 20 ms., since responses    *
*   are expected in much less.  SERVER loop results in   *
*   "blind" time of less than 0.04 ms. per iteration.    *
*                                                        *
*   Changed CALLSRV to pass parameters A and X passed    *
*   in 'rbuf+len' bytes.                                 *
*                                                        *
*   Factored RCVDACK code out for general use.           *
*                                                        *
*   Changed REQUEST to return on ACK or NAK response.    *
*                                                        *
*   03/05/04:                                            *
*                                                        *
*   Changed code to avoid address modification (to allow *
*   code to run in ROM).  This adds one cycle/byte.      *
*                                                        *
*   Changed RCVPKT digital PLL back to +/- 2 cycles/byte *
*   because of page crossing variances in SENDPKT and    *
*   RCVPKT LDA and STA ops that had been overlooked.     *
*                                                        *
*   09/04/03:                                            *
*                                                        *
*   Changed SENDPKT and RCVPKT to new bit timing by      *
*   unrolling loops.                                     *
*                                                        *
*   Changed RCVPKT digital PLL to be +/- 1 cycle/byte    *
*   instead of +/- 2 cycles/byte to tighten tolerances.  *
*                                                        *
**********************************************************
         pag
